package main

import (
	. "code.squareroundforest.org/arpio/wand"
	"code.squareroundforest.org/arpio/wand/tools"
)

var version = "dev"

func main() {
	docreflect := Command("docreflect", tools.Docreflect)
	man := Command("manpages", tools.Man)
	md := Command("markdown", tools.Markdown)
	exec := Command("exec", tools.Exec)
	wand := Group("wand", docreflect, man, md, Default(exec))
	wand = Version(wand, version)
	wand = ShortForm(wand, "f", "no-cache", "c", "clear-cache", "d", "cache-dir")
	conf := MergeConfig(Etc(), UserConfig())
	Exec(wand, conf)
}
